@@ -19,7 +19,7 @@ import sys
 import importlib
 from utils.text import convertStrListDict, _toUNICODE, formatListOr
 from utils.files import pathJoin,readFile
-from utils.portage import searchProfile
+from utils.portage import searchProfile, RepositorySubstituting
 from os import path
 import os
 from collections import OrderedDict
@@ -29,8 +29,9 @@ import pyinotify
 from itertools import *
 import operator
 import traceback
+from contextlib import contextmanager
 from types import StringTypes
-from calculate.lib.configparser import ConfigParser
+from calculate.lib.configparser import ConfigParser, ParsingError
 
 def addStdConfig(fileLst,prefix='/'):
     stdPath = path.join(prefix,
@@ -55,6 +56,9 @@ class CriticalError(Exception):
 class DataVarsError(Exception):
     """Exception of getting variable values"""
 
+class ImportModuleDataVarsError(DataVarsError):
+    """Ошибка импортирования модуля"""
+
 class VariableError(Exception):
     """Exception of sended by Variable"""
 
@@ -91,6 +95,11 @@ class Variable:
       section : string
         section of variable (main,install,builder and etc)
     """
+    class Types:
+        String = "string"
+        List = "list"
+        Choice = "choice"
+        Table = "table"
     # label of variable
     label = None
     # opts for cmdline
@@ -177,6 +186,12 @@ class Variable:
         else:
             return "s"
 
+    def GetBool(self, varname):
+        return self.isTrue(self.Get(varname))
+
+    def GetInteger(self, varname):
+        return int(self.Get(varname))
+
     def Get(self,varname=None,humanreadable=False):
         """Get value of other value"""
         if not varname or varname == self.name:
@@ -207,7 +222,10 @@ class Variable:
 
     def Select(self,selField,**kwargs):
         """Select by datavars"""
-        return self.parent.Select(selField,zipVars=self.ZipVars,**kwargs)
+        return self.parent.Select(selField, zipVars=self.ZipVars,**kwargs)
+
+    def select(self, *fields, **kw):
+        return self.parent.select(*fields, zip_vars=self.ZipVars, **kw)
 
     def getRemoteInfo(self, envFile):
         """getRemoteInfo by datavars"""
@@ -301,6 +319,7 @@ class Variable:
                     oldModeGet = self.modeGet
                     self.modeGet = Variable.NONE
                     value = self.get()
+                    self.untrusted = self.__class__.untrusted
                 finally:
                     self.modeGet = oldModeGet
             # if value not equal previous value
@@ -503,6 +522,7 @@ class Variable:
                 self.invalid = True
                 for var in self.reqVars:
                     var.invalidate()
+                    var.untrusted = var.__class__.untrusted
                 for var in self.reqCheck:
                     var.untrusted = True
                 self.reqVars = []
@@ -515,7 +535,7 @@ class Variable:
     def isTrue(self,value):
         if type(value) == bool:
             return value
-        if  value.lower() in ('yes','on','true'):
+        if value.lower() in ('yes','on','true'):
             return True
         return False
 
@@ -768,14 +788,16 @@ class FileVariable(Variable):
             self.notifier.read_events()
             self.parent.Invalidate(self.name)
 
-class SimpleDataVars:
+class SimpleDataVars(object):
     """
     Simple datavars for temporary link var objs.
     """
     systemRoot = "/"
-    def __init__(self,*objs):
+
+    def __init__(self, *objs):
         self.allVars = {}
         self.cache = {}
+        self.changed = set()
         for obj in objs:
             obj.parent = self
             self.allVars[obj.getVariableName()] = obj
@@ -786,24 +808,70 @@ class SimpleDataVars:
             if self.allVars.get(varname) is not None:
                 self.cache[varname] = self.allVars.get(varname).get()
             else:
+                # ошибка возникает, когда такой переменной нет
+                # в этом пространстве переменных, например
+                # нет зависимой переменной
                 raise DataVarsError(_("Failed to get value %s")%varname)
         return self.cache[varname]
 
+    def Set(self, varname, value):
+        varname = varname.rpartition(".")[2]
+        self.cache[varname] = value
+        self.changed.add(varname)
+
+    def Invalidate(self, varname):
+        varname = varname.rpartition(".")[2]
+        if varname in self.cache:
+            self.cache.pop(varname)
+        if varname in self.changed:
+            self.changed.remove(varname)
+
+    def GetBool(self,varname):
+        return Variable.isTrue(self.Get(varname))
+
+    def GetInteger(self,varname):
+        """
+        Получить целочисленное значение
+        """
+        try:
+            return int(self.Get(varname))
+        except ValueError:
+            return 0
+
+
     def getInfo(self,varname):
         return self.allVars.get(varname, None)
 
-    def flIniFileFrom(self,iniFile):
+    def __getitem__(self, item):
+        return self.Get(item)
+
+    def __setitem__(self, key, value):
+        self.Set(key, value)
+
+    def prepare_all(self):
+        for var in self.allVars.keys():
+            self.Get(var)
+
+    def flIniFileFrom(self,iniFile, system_root=''):
         """
         Read variable values from ini files.
         Return False if ini file is invalid.
         """
         config = ConfigParser(strict=False)
         if path.isdir(iniFile):
-            config.read(addStdConfig(
-                    searchProfile(iniFile, 'calculate.env'),
+            rs = RepositorySubstituting(self, system_root=system_root)
+            try:
+                config.read(addStdConfig(
+                    searchProfile(iniFile, 'calculate.env',
+                                  repository_sub=rs),
                     prefix=self.systemRoot), encoding="utf-8")
+            except ParsingError:
+                raise DataVarsError(_("Error in calculate.env in profile"))
         else:
-            config.read(iniFile, encoding="utf-8")
+            try:
+                config.read(iniFile, encoding="utf-8")
+            except ParsingError:
+                raise DataVarsError(_("Error in %s") % iniFile)
         importVars = {}
         for varname, varobj in self.allVars.items():
             if varobj.section not in importVars:
@@ -811,10 +879,10 @@ class SimpleDataVars:
                     (dict(config.items(varobj.section, raw=True))
                     if config.has_section(varobj.section)
                     else {})
-            if not varname in self.cache:
+            if not varname in self.changed:
                 val = self.unserialize(varobj.type or "string",
                     importVars[varobj.section].get(varname,'').encode('utf-8'))
-                if val or varname in importVars[varobj.section]:
+                if val and val != [[]] or varname in importVars[varobj.section]:
                     self.cache[varname] = val
 
     @staticmethod
@@ -854,6 +922,121 @@ class SimpleDataVars:
             return map(getList(':'), value.split(','))
         return fixEmpty(value).strip("'").strip('"')
 
+    def ZipVars(self,*argvVarNames):
+        """
+        Get zipped values of variables specified by list 'argvVarNames'
+        """
+        return zip(*map(self.Get,argvVarNames))
+
+    def select(self, *fields, **kw):
+        """
+        Новая функция выборки из табличных переменных
+        :param fields: необходимые поля
+        :param kw: аргумены выборки
+        :return:
+        """
+        keywords = ["limit", "zip_vars"]
+        zip_vars = kw.get("zip_vars", self.ZipVars)
+        filter_chain = []
+        variables = list(fields)
+        for k in (x for x in kw if x not in keywords):
+            varname, op, funcname = k.partition("__")
+            if varname[2] != "_":
+                varname = varname.replace("_", ".", 1)
+            if not op:
+                funcname = "eq"
+            variables.append(varname)
+            func_maps = {
+                'eq': lambda x, y: lambda z: z[x] == y,
+                'ne': lambda x, y: lambda z: z[x] != y,
+                'in': lambda x, y: lambda z: z[x] in y,
+                'not_in': lambda x, y: lambda z: z[x] not in y,
+            }
+            filter_chain.append(func_maps[funcname](len(variables) - 1, kw[k]))
+        l = len(fields)
+        if l == 1:
+            ret_func = lambda x:x[0]
+        else:
+            ret_func = lambda x:x[:l]
+        def generator(limit):
+            c = 0
+            for x in zip_vars(*variables):
+                for f in filter_chain:
+                    if not f(x):
+                        break
+                else:
+                    yield ret_func(x)
+                    if limit and c >= limit:
+                        break
+                    else:
+                        c += 1
+
+        limit = kw.get("limit", None)
+        if limit == 1:
+            for x in generator(limit):
+                return x
+            else:
+                if l == 1:
+                    return ""
+                else:
+                    return [""]*l
+        else:
+            return list(generator(limit))
+
+
+    def Select(self,selField,where="os_disk_dev",eq=None,ne=None,
+               _in=None,_notin=None,like=None,notlike=None,func=None,
+               sort=None, sortkey=None, limit=None,zipVars=None):
+        """Select value from table variables"""
+        if zipVars is None:
+            zipVars = self.ZipVars
+        if func:
+            if func.func_code.co_argcount > 1:
+                filterFunc = lambda x:func(*x[:func.func_code.co_argcount])
+            else:
+                filterFunc = func
+        elif eq != None:
+            filterFunc = lambda x:x[0] == eq
+        elif ne != None:
+            filterFunc = lambda x:x[0] != ne
+        elif _in != None:
+            filterFunc = lambda x:x[0] in _in
+        elif _notin != None:
+            filterFunc = lambda x:not x[0] in _notin
+        elif like != None:
+            filterFunc = lambda x:re.compile(like).search(x[0])
+        elif notlike != None:
+            filterFunc = lambda x:not re.compile(notlike).search(x[0])
+        else:
+            filterFunc = lambda x:x
+
+        if not type(where) in (tuple,list):
+            where = [where]
+        woffset = len(where)
+        if type(selField) in (tuple,list):
+            count = len(selField)+woffset
+            mapFunc = lambda x:x[woffset:]
+            res = filter(filterFunc,
+                         zipVars(*(where+selField)))
+        else:
+            count = 1+woffset
+            fields = where+[selField]
+            mapFunc = lambda x:x[woffset]
+            res = filter(filterFunc,
+                         zipVars(*fields))
+        if sort:
+            if "/" in sort:
+                sort,sortkey = sort.split('/')
+                sortkey = itemgetter(*map(int,sortkey.split(',')))
+            res.sort(key=sortkey,reverse=True if sort == "DESC" else False)
+        if limit == 1:
+            if not any(res):
+                res = [[""]*(count)]
+            return mapFunc(res[0])
+        else:
+            return map(mapFunc,res[:limit])
+
+
 
 class DataVars(SimpleDataVars):
     """
@@ -888,6 +1071,16 @@ class DataVars(SimpleDataVars):
         self.importedModules = []
         self.iniCache = {}
 
+    @contextmanager
+    def useDefaultModule(self, modulename):
+        oldmodule = self.defaultModule
+        try:
+            self.defaultModule = modulename
+            yield self
+        finally:
+            self.defaultModule = oldmodule
+
+
     def isModuleInstalled(self,module):
         try:
             importlib.import_module("calculate.%s.datavars"%module)
@@ -940,10 +1133,14 @@ class DataVars(SimpleDataVars):
         return section
 
     def raiseVariableNotFound(self,varname,*args,**kwargs):
+        # TODO: DEBUG raise variable not found
+        # import traceback
+        # traceback.print_stack()
+        # print "Variable not found:", varname
         raise DataVarsError(_("Variable %s not found")%varname)
 
     def raiseModuleError(self,module,error,*args,**kwargs):
-        raise DataVarsError("\n".join([
+        raise ImportModuleDataVarsError("\n".join([
             _("Failed to import module %s")%module,
             _("error")  + ": " +str(error)]))
 
@@ -960,7 +1157,7 @@ class DataVars(SimpleDataVars):
             if varname in self.allVars.keys():
                 section,varObj = self.allVars[varname]
                 newVar = varObj(parent=self,section=section)
-                if hasattr(newVar,"refresh"):
+                if hasattr(newVar, "refresh"):
                     self.refresh.append(newVar)
                 self.loadVariables[varname] = newVar
             else:
@@ -1002,10 +1199,12 @@ class DataVars(SimpleDataVars):
         for iniFile in calculateIniFiles:
             if path.exists(iniFile):
                 config = ConfigParser(strict=False)
-                config.read(iniFile, encoding="utf-8")
                 try:
+                    config.read(iniFile, encoding="utf-8")
                     value = config[section][varname]
                     retVal = value
+                except ParsingError:
+                    raise DataVarsError(_("Error in %s") % iniFile)
                 except KeyError:
                     pass
         return retVal.encode('UTF-8')
@@ -1046,8 +1245,9 @@ class DataVars(SimpleDataVars):
                     return True
                 raise CriticalError(str(e))
             if os.path.exists(make_profile):
+                profiledir = path.dirname(make_profile)
                 profiles = [('profile',
-                            path.join('/etc',os.readlink(make_profile)))]
+                            path.join(profiledir,os.readlink(make_profile)))]
             else:
                 profiles = []
             for iniName,iniFile in profiles+\
@@ -1055,11 +1255,21 @@ class DataVars(SimpleDataVars):
                 if path.exists(iniFile) or iniName == 'profile':
                     config = ConfigParser(strict=False)
                     if iniName == 'profile':
-                        config.read(addStdConfig(
-                                    searchProfile(iniFile,'calculate.env')),
+                        repos = RepositorySubstituting(self)
+                        try:
+                            config.read(addStdConfig(
+                                    searchProfile(iniFile,'calculate.env',
+                                                  repository_sub=repos)),
                                     encoding="utf-8")
+                        except ParsingError:
+                            raise DataVarsError(
+                                _("Error in profile calculate.env"))
                     else:
-                        config.read(iniFile, encoding="utf-8")
+                        try:
+                            config.read(iniFile, encoding="utf-8")
+                        except ParsingError:
+                            raise DataVarsError(
+                                _("Error in %s")%iniFile)
                     iniSections = config.sections()
                     if not iniSections:
                         continue
@@ -1084,18 +1294,6 @@ class DataVars(SimpleDataVars):
             DataVars.l.release()
         return var
 
-    def GetBool(self,varname):
-        return Variable.isTrue(self.Get(varname))
-
-    def GetInteger(self,varname):
-        """
-        Получить целочисленное значение
-        """
-        try:
-            return int(self.Get(varname))
-        except ValueError:
-            return 0
-
     def isFromIni(self,varname):
         """
         Check what value get from ini file
@@ -1123,7 +1321,12 @@ class DataVars(SimpleDataVars):
             section,varname = self.splitVarname(varname)
             if not varname in self.loadVariables or \
                 not section in self.importedModules:
-                self.loadVariable(varname,section=section)
+                try:
+                    self.loadVariable(varname,section=section)
+                except ImportModuleDataVarsError:
+                    if varname.startswith("ac_"):
+                        return ""
+                    raise
             varObj = self.loadVariables[varname]
 
             # check section
@@ -1336,69 +1539,11 @@ class DataVars(SimpleDataVars):
                                   "template variables path")%location)
         return False
 
-    def Select(self,selField,where="os_disk_dev",eq=None,ne=None,
-               _in=None,_notin=None,like=None,notlike=None,func=None,
-               sort=None, sortkey=None, limit=None,zipVars=None):
-        """Select value from table variables"""
-        if zipVars is None:
-            zipVars = self.ZipVars
-        if func:
-            if func.func_code.co_argcount > 1:
-                filterFunc = lambda x:func(*x[:func.func_code.co_argcount])
-            else:
-                filterFunc = func
-        elif eq != None:
-            filterFunc = lambda x:x[0] == eq
-        elif ne != None:
-            filterFunc = lambda x:x[0] != ne
-        elif _in != None:
-            filterFunc = lambda x:x[0] in _in
-        elif _notin != None:
-            filterFunc = lambda x:not x[0] in _notin
-        elif like != None:
-            filterFunc = lambda x:re.compile(like).search(x[0])
-        elif notlike != None:
-            filterFunc = lambda x:not re.compile(notlike).search(x[0])
-        else:
-            filterFunc = lambda x:x
-
-        if not type(where) in (tuple,list):
-            where = [where]
-        woffset = len(where)
-        if type(selField) in (tuple,list):
-            count = len(selField)+woffset
-            mapFunc = lambda x:x[woffset:]
-            res = filter(filterFunc,
-                  zipVars(*(where+selField)))
-        else:
-            count = 1+woffset
-            fields = where+[selField]
-            mapFunc = lambda x:x[woffset]
-            res = filter(filterFunc,
-                  zipVars(*fields))
-        if sort:
-            if "/" in sort:
-                sort,sortkey = sort.split('/')
-                sortkey = itemgetter(*map(int,sortkey.split(',')))
-            res.sort(key=sortkey,reverse=True if sort == "DESC" else False)
-        if limit == 1:
-            if not any(res):
-                res = [[""]*(count)]
-            return mapFunc(res[0])
-        else:
-            return map(mapFunc,res[:limit])
-
     def processRefresh(self):
         for refreshVar in self.refresh:
             if not refreshVar.invalid:
                 refreshVar.refresh()
 
-    def ZipVars(self,*argvVarNames):
-        """
-        Get zipped values of variables specified by list 'argvVarNames'
-        """
-        return zip(*map(self.Get,argvVarNames))
-
     def close(self):
         for varObj in self.loadVariables.values():
             if hasattr(varObj,"close"):
@@ -1444,7 +1589,8 @@ class DataVars(SimpleDataVars):
              'next_label':next_label or default_next_label})
         lastIndex = len(self.groups)-1
         for var in chain(normal,expert):
-            self.mapVarGroup[var] = lastIndex
+            varname = var.rpartition('.')[2]
+            self.mapVarGroup[varname] = lastIndex
         return self
 
     def getGroups(self):
